#! /usr/bin/env python2 

import os
import sys
import re
import argparse
import copy
import subprocess
__doc__ = """ cure the path var provided as arg1
              its a colon separated list of unique pathes.
               we get rid of non existing pathes and doublons,
               and output the result
          """

slash = os.path.sep
col = column = ':'

def vireDoublons(pathes):
    dejavu = set([])
    for path in pathes:
        if path in dejavu: continue
        dejavu.add(path)
        yield path

def vireNonexistent(pathes):
    return (path for path in pathes if os.path.isdir(path))


def multislash(pathes):
    ms = re.compile(slash + '+')
    for path in pathes:
        monoslashed = ms.sub(slash, path, count = 88)
        if slash == monoslashed[-1]: yield monoslashed[:-1]
        else: yield monoslashed

def normpath(pathes):
    for path in pathes:
        if path:
            normed = os.path.normpath(os.path.abspath(path))
            yield normed

#================================================================
# test routines
#================================================================
def mkdirs(levels, root):
    """ prepare dirs intended for test, to be removed after test
    levels is a list of strings to be split, to form dirs at some level
"""
    os.mkdir(root)
    dirs = [root]
    for level in levels:
        # each level corresponds to a depth in dir hierarchy.
        newdirs = []
        leaves = level.split()  # that is: leaves at this level!
        newdirs =   [os.path.join(dir, leaf) for dir in dirs for leaf in leaves]
        for newdir in newdirs:
            os.mkdir(newdir)
        dirs = newdirs
    return dirs

def testMkdirs():
    """ specific test of the mkdirs() function.
        read this code to get what mkdirs() makes
    """
    tmpname = os.tmpnam()

    dirs = mkdirs( 'a b c:1 2 3 4'.split(col),tmpname)
    assert len(dirs) == 3 * 4
    for dir in dirs:
        assert os.path.exists(dir)
    ends = set([x[-1] for x in dirs])
    assert ends == set('1 2 3 4'.split())
    tips = [x[-3:] for x in dirs] # keep 3 last: a/1, ...
    assert 'a/1' in tips
    assert 'b/3' in tips
    # cleanup:
    subprocess.Popen('rm -rf'.split() + dirs)
    
    
def test():
    tmpname = os.tmpnam()

    dirs = mkdirs( 'a b c:1 2 3 4'.split(col),tmpname)
    nonex = os.path.join(dirs[0], 'toto')
    pathes = dirs[:5]           # just a list of existing pathes.
    # exhaustive doublon removal test:
    # insert in each possible position in tampered (copy of pathes),
    # each possible path.
    # show each time doublon removal.
    for i in range(len(pathes)):
        for extra in pathes:
            tampered = copy.copy(pathes)
            tampered.insert(i, extra)
            cured = [path for path in vireDoublons(tampered)]
            # order may have changed for the extra path:
            assert set(cured) == set(pathes)
            # order unchanged for other pathes
            assert [x for x in cured if x != extra] ==  [x for x in pathes if x != extra] 

    rc = subprocess.call('rm -rf'.split() + [tmpname])
    assert rc == 0, 'cud not remove %s, rm code %d' %(tmpname, rc)

    # double slash removal test:
    import overslasher
    ''' pathes is a list of path(es).
        tbc...
        '''
    pathes = 'a/b/c 1/3/2/4 gi/gu/la/to/mu/fi'.split()
    tamperedGen = overslasher.overslasher(pathes, 2)
    cured = [[x for x in multislash(tampered)] for tampered in tamperedGen]
    different = [ x for x in cured if x != pathes]
    assert len(different) == 0

    # bug: trailing column makes an empty path, transformed by normpath
    # in current working dir: normpath shud discard empty pathes before
    # normalizing.
    dirs = mkdirs( 'a b c:1 2 3 4'.split(col),tmpname)
    assert len(dirs) == 12
    ps = col.join(dirs)
    tampered = ps + col
    tamDirs = tampered.split(col)
    assert ps.split(col) == dirs
    normed = list(normpath(dirs))
    assert normed == dirs
    tamNormed = [x for x in normpath(tamDirs)]
    assert tamNormed == dirs, 'empty dir promoted to a real path'

#================================================================
# main
#================================================================
def main():
    parser = argparse.ArgumentParser()
    parser.add_argument("-v",
                        "--verbose",
                        help="increase output verbosity",
                        action="store_true")
    parser.add_argument("-l",
                        "--list",
                        help="output path as a list (one per line)",
                        action="store_true")
    parser.add_argument("-e",
                        "--eval",
                        help="output path ready to eval",
                        action="store_true")
    parser.add_argument("varname")

    global args
    args = parser.parse_args()
    if args.list and args.eval:
        print >> sys.stderr, "-l and -e are exclusive"
        sys.exit(11)
    if not args.list and not args.eval:
        args.list = True

    try:
        pathes = os.environ[args.varname].split(col)
    except KeyError:
        print >> sys.stderr, "%s not defined" % args.varname
        sys.exit(11)
    cured = [x for x in vireNonexistent(vireDoublons(multislash(normpath(pathes))))]
    if args.eval:
        print "%s=%s" % (args.varname, col.join(cured))
    if args.list:
        for path in cured:
            print path


if __name__ == '__main__':
    main()
